@@ -67,6 +67,7 @@ struct _CcBackgroundPanelPrivate
 
   GnomeWPItem *current_background;
   gboolean current_source_readonly;
+  gint current_source;
 
   GCancellable *copy_cancellable;
 
@@ -220,7 +221,58 @@ cc_background_panel_class_init (CcBackgroundPanelClass *klass)
 static void
 cc_background_panel_class_finalize (CcBackgroundPanelClass *klass)
 {
+}
 
+static void
+source_update_edit_box (CcBackgroundPanelPrivate *priv)
+{
+  if (priv->current_source == SOURCE_COLORS)
+    {
+      gtk_widget_hide (WID ("style-combobox"));
+      gtk_widget_show (WID ("style-pcolor"));
+
+      if (priv->current_background &&
+	  priv->current_background->shade_type == G_DESKTOP_BACKGROUND_SHADING_SOLID)
+        gtk_widget_hide (WID ("style-scolor"));
+      else
+        gtk_widget_show (WID ("style-scolor"));
+    }
+  else
+    {
+      if (!priv->current_source_readonly)
+        {
+          gtk_widget_show (WID ("style-pcolor"));
+          gtk_widget_hide (WID ("style-scolor"));
+          gtk_widget_show (WID ("style-combobox"));
+	}
+      else
+        {
+          gtk_widget_hide (WID ("style-pcolor"));
+          gtk_widget_hide (WID ("style-scolor"));
+          gtk_widget_hide (WID ("style-combobox"));
+	}
+    }
+}
+
+static void
+setup_edit_box (CcBackgroundPanelPrivate *priv)
+{
+  g_assert (priv->current_background);
+
+  if (g_str_equal (priv->current_background->filename, "(none)"))
+    {
+      gtk_widget_hide (WID ("style-combobox"));
+      gtk_widget_show (WID ("style-pcolor"));
+
+      if (priv->current_background->shade_type == G_DESKTOP_BACKGROUND_SHADING_SOLID)
+        gtk_widget_hide (WID ("style-scolor"));
+      else
+        gtk_widget_show (WID ("style-scolor"));
+    }
+  else
+    {
+      /* FIXME other cases */
+    }
 }
 
 static void
@@ -279,12 +331,43 @@ copy_finished_cb (GObject      *source_object,
   g_object_unref (panel);
 }
 
+static void
+select_style (GtkComboBox *box,
+	      GDesktopBackgroundStyle new_style)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gboolean cont;
+
+  model = gtk_combo_box_get_model (box);
+  cont = gtk_tree_model_get_iter_first (model, &iter);
+  while (cont != FALSE)
+    {
+      GDesktopBackgroundStyle style;
+
+      gtk_tree_model_get (model, &iter,
+			  1, &style,
+			  -1);
+
+      if (style == new_style)
+        {
+          gtk_combo_box_set_active_iter (box, &iter);
+          break;
+	}
+      cont = gtk_tree_model_iter_next (model, &iter);
+    }
+
+  if (cont == FALSE)
+    gtk_combo_box_set_active (box, -1);
+}
+
 static void
 update_preview (CcBackgroundPanelPrivate *priv,
                 GnomeWPItem              *item,
                 gboolean                  redraw_preview)
 {
   gchar *markup;
+  gboolean changes_with_time;
 
   if (item && priv->current_background)
     {
@@ -296,8 +379,6 @@ update_preview (CcBackgroundPanelPrivate *priv,
         gdk_color_free (priv->current_background->scolor);
       priv->current_background->scolor = gdk_color_copy (item->scolor);
 
-
-
       g_free (priv->current_background->filename);
       priv->current_background->filename = g_strdup (item->filename);
 
@@ -305,13 +386,18 @@ update_preview (CcBackgroundPanelPrivate *priv,
       priv->current_background->name = g_strdup (item->name);
 
       priv->current_background->options = item->options;
+      priv->current_background->shade_type = item->shade_type;
+
+      priv->current_background->height = item->height;
+      priv->current_background->width = item->width;
+
+      gnome_wp_item_ensure_gnome_bg (priv->current_background);
+      gnome_wp_item_update_size (priv->current_background, priv->thumb_factory);
     }
 
+  source_update_edit_box (priv);
 
-  if (!priv->current_source_readonly)
-    gtk_widget_show (WID ("edit-hbox"));
-  else
-    gtk_widget_hide (WID ("edit-hbox"));
+  changes_with_time = FALSE;
 
   if (priv->current_background)
     {
@@ -319,13 +405,23 @@ update_preview (CcBackgroundPanelPrivate *priv,
       gtk_label_set_markup (GTK_LABEL (WID ("background-label")), markup);
       g_free (markup);
 
-      gtk_color_button_set_color (GTK_COLOR_BUTTON (WID ("style-color")),
-                                  priv->current_background->pcolor);
+      gtk_label_set_text (GTK_LABEL (WID ("size_label")), priv->current_background->size);
 
-      gtk_combo_box_set_active (GTK_COMBO_BOX (WID ("style-combobox")),
-                                priv->current_background->options);
+      gtk_color_button_set_color (GTK_COLOR_BUTTON (WID ("style-pcolor")),
+                                  priv->current_background->pcolor);
+      gtk_color_button_set_color (GTK_COLOR_BUTTON (WID ("style-scolor")),
+                                  priv->current_background->scolor);
+
+      select_style (GTK_COMBO_BOX (WID ("style-combobox")),
+                    priv->current_background->options);
+
+      if (priv->current_background->bg)
+        changes_with_time = gnome_bg_changes_with_time (priv->current_background->bg);
     }
 
+  gtk_widget_set_visible (WID ("slide_image"), changes_with_time);
+  gtk_widget_set_visible (WID ("slide-label"), changes_with_time);
+
   if (redraw_preview)
     gtk_widget_queue_draw (WID ("preview-area"));
 }
@@ -352,8 +448,9 @@ backgrounds_changed_cb (GtkIconView       *icon_view,
   model = gtk_combo_box_get_model (GTK_COMBO_BOX (WID ("sources-combobox")));
   gtk_combo_box_get_active_iter (GTK_COMBO_BOX (WID ("sources-combobox")),
                                  &iter);
-  gtk_tree_model_get (model, &iter, COL_SOURCE_READONLY, &priv->current_source_readonly, -1);
-
+  gtk_tree_model_get (model, &iter,
+  		      COL_SOURCE_READONLY, &priv->current_source_readonly,
+  		      COL_SOURCE_TYPE, &priv->current_source, -1);
 
   model = gtk_icon_view_get_model (icon_view);
 
@@ -364,9 +461,6 @@ backgrounds_changed_cb (GtkIconView       *icon_view,
 
   gtk_tree_model_get (model, &iter, 1, &item, -1);
 
-  /* Do all changes in one 'transaction' */
-  g_settings_delay (priv->settings);
-
   if (!g_strcmp0 (item->filename, "(none)"))
     {
       g_settings_set_enum (priv->settings, WP_OPTIONS_KEY, G_DESKTOP_BACKGROUND_STYLE_NONE);
@@ -452,10 +546,23 @@ backgrounds_changed_cb (GtkIconView       *icon_view,
 
   g_settings_set_enum (priv->settings, WP_SHADING_KEY, item->shade_type);
 
-  pcolor = gdk_color_to_string (item->pcolor);
-  scolor = gdk_color_to_string (item->scolor);
-  g_settings_set_string (priv->settings, WP_PCOLOR_KEY, pcolor);
-  g_settings_set_string (priv->settings, WP_SCOLOR_KEY, scolor);
+  /* When changing for another colour, don't overwrite what's
+   * in GSettings, but read from it instead */
+  if (priv->current_source == SOURCE_COLORS)
+    {
+      pcolor = g_settings_get_string (priv->settings, WP_PCOLOR_KEY);
+      scolor = g_settings_get_string (priv->settings, WP_SCOLOR_KEY);
+      gdk_color_parse (pcolor, item->pcolor);
+      gdk_color_parse (scolor, item->scolor);
+    }
+  else
+    {
+      pcolor = gdk_color_to_string (item->pcolor);
+      scolor = gdk_color_to_string (item->scolor);
+      g_settings_set_string (priv->settings, WP_PCOLOR_KEY, pcolor);
+      g_settings_set_string (priv->settings, WP_SCOLOR_KEY, scolor);
+    }
+
   g_free (pcolor);
   g_free (scolor);
 
@@ -485,10 +592,13 @@ preview_draw_cb (GtkWidget         *widget,
 
   if (priv->current_background)
     {
-      pixbuf = gnome_wp_item_get_thumbnail (priv->current_background,
-                                            priv->thumb_factory,
-                                            preview_width,
-                                            preview_height);
+      GIcon *icon;
+      icon = gnome_wp_item_get_frame_thumbnail (priv->current_background,
+                                                priv->thumb_factory,
+                                                preview_width,
+                                                preview_height,
+                                                -2);
+      pixbuf = GDK_PIXBUF (icon);
     }
 
   if (!priv->display_base)
@@ -544,7 +654,7 @@ style_changed_cb (GtkComboBox       *box,
   CcBackgroundPanelPrivate *priv = panel->priv;
   GtkTreeModel *model;
   GtkTreeIter iter;
-  gint value;
+  GDesktopBackgroundStyle value;
 
   if (!gtk_combo_box_get_active_iter (box, &iter))
     {
@@ -558,7 +668,9 @@ style_changed_cb (GtkComboBox       *box,
   g_settings_set_enum (priv->settings, WP_OPTIONS_KEY, value);
 
   if (priv->current_background)
-    priv->current_background->options = gtk_combo_box_get_active (box);
+    priv->current_background->options = value;
+
+  g_settings_apply (priv->settings);
 
   update_preview (priv, NULL, TRUE);
 }
@@ -570,16 +682,28 @@ color_changed_cb (GtkColorButton    *button,
   CcBackgroundPanelPrivate *priv = panel->priv;
   GdkColor color;
   gchar *value;
+  gboolean is_pcolor = FALSE;
 
   gtk_color_button_get_color (button, &color);
+  if (WID ("style-pcolor") == GTK_WIDGET (button))
+    is_pcolor = TRUE;
 
   if (priv->current_background)
-    *priv->current_background->pcolor = color;
+    {
+      if (is_pcolor)
+        *priv->current_background->pcolor = color;
+      else
+        *priv->current_background->scolor = color;
+    }
 
   value = gdk_color_to_string (&color);
 
-  g_settings_set_string (priv->settings, WP_PCOLOR_KEY, value);
-  g_settings_set_string (priv->settings, WP_SCOLOR_KEY, value);
+  if (is_pcolor)
+    g_settings_set_string (priv->settings, WP_PCOLOR_KEY, value);
+  else
+    g_settings_set_string (priv->settings, WP_SCOLOR_KEY, value);
+
+  g_settings_apply (priv->settings);
 
   g_free (value);
 
@@ -613,6 +737,7 @@ cc_background_panel_init (CcBackgroundPanel *self)
     }
 
   priv->settings = g_settings_new (WP_PATH_ID);
+  g_settings_delay (priv->settings);
 
   store = (GtkListStore*) gtk_builder_get_object (priv->builder,
                                                   "sources-liststore");
@@ -635,9 +760,9 @@ cc_background_panel_init (CcBackgroundPanel *self)
 
   priv->colors_source = bg_colors_source_new ();
   gtk_list_store_insert_with_values (store, NULL, G_MAXINT,
-                                     COL_SOURCE_NAME, _("Colors"),
+                                     COL_SOURCE_NAME, _("Colors & Gradients"),
                                      COL_SOURCE_TYPE, SOURCE_COLORS,
-                                     COL_SOURCE_READONLY, TRUE,
+                                     COL_SOURCE_READONLY, FALSE,
                                      COL_SOURCE, priv->colors_source,
                                      -1);
 
@@ -686,7 +811,9 @@ cc_background_panel_init (CcBackgroundPanel *self)
   g_signal_connect (WID ("style-combobox"), "changed",
                     G_CALLBACK (style_changed_cb), self);
 
-  g_signal_connect (WID ("style-color"), "color-set",
+  g_signal_connect (WID ("style-pcolor"), "color-set",
+                    G_CALLBACK (color_changed_cb), self);
+  g_signal_connect (WID ("style-scolor"), "color-set",
                     G_CALLBACK (color_changed_cb), self);
 
   priv->copy_cancellable = g_cancellable_new ();
@@ -703,12 +830,16 @@ cc_background_panel_init (CcBackgroundPanel *self)
 
   priv->current_background = g_new0 (GnomeWPItem, 1);
   priv->current_background->filename = filename;
-  priv->current_background->name = g_strdup ("");
+  priv->current_background->name = g_strdup (_("Current background"));
 
   gnome_wp_item_update (priv->current_background);
   gnome_wp_item_ensure_gnome_bg (priv->current_background);
+  gnome_wp_item_update_size (priv->current_background, priv->thumb_factory);
 
   update_preview (priv, NULL, TRUE);
+
+  /* Setup the edit box with our current settings */
+  setup_edit_box (priv);
 }
 
 void
